# The LearningOnline Network with CAPA 
# implements the tags that control the hints
#
# $Id: hint.pm,v 1.27 2002/03/06 20:39:15 sakharuk Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::hinttags; 

use strict;
use Apache::lonnet;
use capa;

BEGIN {
  &Apache::lonxml::register('Apache::hinttags',('hintgroup','hintpart','numericalhint'));
}


@Apache::hint::which=();
sub start_hintgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $skiptoend='0';
  my $result;

  if ($target eq 'web') {
    my $id=$Apache::inputtags::part;
    my $numtries=$Apache::lonhomework::history{"resource.$id.tries"};
    if ( $numtries eq '') { $numtries = 0; }
    my $hinttries=&Apache::lonnet::EXT("resource.$id.hinttries");
    if ( $hinttries eq '') { $hinttries = 1; }
    &Apache::lonxml::debug("found :$id:$numtries:$hinttries:");
    if ( $numtries < $hinttries ) {
      $skiptoend='1';
    } else {
      if ($target eq 'web') {$result='<table bgcolor="#dddddd"><tr><td>';}
    }
    if ($skiptoend) {
      &Apache::lonxml::get_all_text("/hintgroup",$$parser[$#$parser]);
    }
  } elsif ($target eq 'tex') {
    $result .= '\keephidden{'; 
  }
  @Apache::hint::which=();
  return $result;
}

sub end_hintgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result;

  if ($target eq 'web') {
    my $id=$Apache::inputtags::part;
    my $numtries=$Apache::lonhomework::history{"resource.$id.tries"};
    if ( $numtries eq '') { $numtries = 0; }
    my $hinttries=&Apache::lonnet::EXT("resource.$id.hinttries");
    if ( $hinttries eq '') { $hinttries = 1; }
    &Apache::lonxml::debug("found :$id:$numtries:$hinttries:");
    if ( $numtries >= $hinttries ) {
      $result='</td></tr></table>';
    }
  } elsif ($target eq 'edit') {
    $result.=&Apache::edit::end_table();
  } elsif ($target eq 'tex') {
    $result .= '}'; 
  }
  @Apache::hint::which=();
  return $result;
}

sub start_numericalhint {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  #do everything in end, so intervening <responseparams> work
  &Apache::response::start_hintresponse($parstack,$safeeval);
  my $result;
  if ($target eq 'edit') {
    $result.=&Apache::edit::tag_start($target,$token);
    $result.=&Apache::edit::text_arg('Name:','name',$token);
    $result.=&Apache::edit::text_arg('Answer:','answer',$token);
    $result.=&Apache::edit::text_arg('Unit:','unit',$token,5);
    $result.=&Apache::edit::text_arg('Format:','format',$token,4);
    $result.=&Apache::edit::end_row();
    $result.=&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=
      &Apache::edit::get_new_args($token,$parstack,$safeeval,
				  'name','answer','unit', 'format');
    if ($constructtag) {
      $result  = &Apache::edit::rebuild_tag($token);
      $result .= &Apache::edit::handle_insert();
    }
  }
  return $result;
}

sub end_numericalhint {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result;
  if ($target eq 'web') {
    $safeeval->share_from('capa',['&caparesponse_capa_check_answer']);
    my $name= &Apache::lonxml::get_param('name',$parstack,$safeeval);
    &Apache::response::setup_params('numericalhint');
    my $partid=$Apache::inputtags::part;
    my $id=$Apache::inputtags::response['-1'];
    #id submissions occured under
    my $submitid=$Apache::inputtags::response['-2'];
    my $response = $Apache::lonhomework::history{
			    "resource.$partid.$submitid.submission"};
    &Apache::lonxml::debug("hintgroup is using $response<br />\n");
    #build safe space expression
    my $expression="&caparesponse_check_list('".$response."','".
      $$parstack[$#$parstack];
    #need to get all possible parms
    foreach my $key (keys(%Apache::inputtags::params)) {
      $expression.= ';my $'. #'
	$key.'="'.$Apache::inputtags::params{$key}.'"';
    }
    $expression.="');";
    $result = &Apache::run::run($expression,$safeeval);
    &Apache::lonxml::debug("$expression:result:$result:$Apache::lonxml::curdepth");
    my ($awards) = split /:/ , $result;
    my ($ad) = &Apache::inputtags::finalizeawards(split /,/ , $awards);
    if ($ad eq 'EXACT_ANS' || $ad eq 'APPROX_ANS') { push (@Apache::hint::which,$name); }
    $result='';
  } elsif ($target eq 'meta') {
    $result=&Apache::response::meta_package_write('numericalhint');
  } elsif ($target eq 'edit') {
    $result.='</td></tr>'.&Apache::edit::end_table;
  }
  &Apache::response::end_hintresponse();
  return $result;
}

# a part shows if it is on, if no specific parts are on, then default shows
sub start_hintpart {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;

  my $show ='0';
  my $result = '';
  if ($target eq 'web') {
    my $on= &Apache::lonxml::get_param('on',$parstack,$safeeval);
    &Apache::lonxml::debug("hintpart sees $on and ,$#Apache::hint::which");
    if ( $on eq 'default' && $#Apache::hint::which == '-1') {
      $show=1;
    } else {
      my $which;
      foreach $which (@Apache::hint::which) { if ($which eq $on) { $show = 1; last } }
    }
    if (!$show) {
      &Apache::lonxml::get_all_text("/hintpart",$$parser[$#$parser]);
    }
  } elsif ($target eq 'grade') {
    &Apache::lonxml::get_all_text("/hintpart",$$parser[$#$parser]);
  } elsif ($target eq 'edit') {
    $result.= &Apache::edit::tag_start($target,$token);
    $result.= &Apache::edit::text_arg('On:','on',$token);
    $result.= &Apache::edit::end_row();
    $result.= &Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,
						 'on');
    if ($constructtag) {
      $result = &Apache::edit::rebuild_tag($token);
      $result.=&Apache::edit::handle_insert();
    }
  }
  return $result;
}

sub end_hintpart {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result;
  if ($target eq 'edit') { $result.=&Apache::edit::end_table; }
  return $result;
}

1;
__END__
