# The LearningOnline Network with CAPA
# The LON-CAPA Homework handler
#
# $Id: lonhomework.pm,v 1.81 2002/06/26 17:20:38 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# Guy Albertelli
# 11/30 Gerd Kortemeyer
# 6/1,8/17,8/18 Gerd Kortemeyer

package Apache::lonhomework;
use strict;
use Apache::style();
use Apache::lonxml();
use Apache::lonnet();
use Apache::lonplot();
use Apache::inputtags();
use Apache::structuretags();
use Apache::randomlabel();
use Apache::response();
use Apache::hint();
use Apache::outputtags();
use Apache::Constants qw(:common);
use HTML::Entities();
#use Time::HiRes qw( gettimeofday tv_interval );

BEGIN {
  &Apache::lonxml::register_insert();
}

sub get_target {
  if ( $ENV{'request.state'} eq "published") {
    if ( defined($ENV{'form.grade_target'}) 
	 && ($Apache::lonhomework::viewgrades == 'F' )) {
      return ($ENV{'form.grade_target'});
    }
    if ( defined($ENV{'form.submitted'})) {
      return ('grade', 'web');
    } else {
      return ('web');
    }
  } elsif ($ENV{'request.state'} eq "construct") {
    if ( defined($ENV{'form.grade_target'}) ) {
      return ($ENV{'form.grade_target'});
    }
    if ( defined($ENV{'form.preview'})) {
      if ( defined($ENV{'form.submitted'})) {
	return ('grade', 'web');
      } else {
	return ('web');
      }
    } else {
      if ( $ENV{'form.problemmode'} eq 'View' ) {
	if ( defined($ENV{'form.submitted'}) &&
	     (!defined($ENV{'form.resetdata'})) ) {
	  return ('grade', 'web','answer');
	} else {
	  return ('web','answer');
	}
      } elsif ( $ENV{'form.problemmode'} eq 'Edit' ) {
	if ( $ENV{'form.submitted'} eq 'edit' ) {
	  if ( $ENV{'form.submit'} eq 'Submit Changes and View' ) {
	    return ('modified','web','answer');
	  } else {
	    return ('modified','edit');
	  }
	} else {
	  return ('edit');
	}
      } else {
	return ('web');
      }
    }
  }
  return ();
}

sub setup_vars {
  my ($target) = @_;
  return ';'
#  return ';$external::target='.$target.';';
}

sub send_header {
  my ($request)= @_;
  $request->print(&Apache::lontexconvert::header());
#  $request->print('<form name='.$ENV{'form.request.prefix'}.'lonhomework method="POST" action="'.$request->uri.'">');
}

sub createmenu {
  my ($which,$request)=@_;
  if ($which eq 'grade') {
    $request->print('<script language="JavaScript"> 
          hwkmenu=window.open("/res/adm/pages/homeworkmenu.html","homeworkremote",
                 "height=350,width=150,menubar=no");
          </script>');
  }
}

sub send_footer {
  my ($request)= @_;
#  $request->print('</form>');
  $request->print(&Apache::lontexconvert::footer());
}

$Apache::lonxml::browse='';

sub check_access {
  my ($id) = @_;
  my $date ='';
  my $status = '';
  my $datemsg = '';
  my $lastdate = '';
  my $temp;
  my $type;
  my $passed;
  &Apache::lonxml::debug("checking for part :$id:");
  &Apache::lonxml::debug("time:".time);
  foreach $temp ("opendate","duedate","answerdate") {
    $lastdate = $date;
    $date = &Apache::lonnet::EXT("resource.$id.$temp");
    &Apache::lonxml::debug("found :$date: for :$temp:");
    if ($date eq '') {
      $date = "an unknown date"; $passed = 0;
    } elsif ($date eq 'con_lost') {
      $date = "an indeterminate date"; $passed = 0;
    } else {
      if (time < $date) { $passed = 0; } else { $passed = 1; }
      $date = localtime $date;
    }
    if (!$passed) { $type=$temp; last; }
  }
  &Apache::lonxml::debug("have :$type:$passed:");
  if ($passed) {
    $status='SHOW_ANSWER';
    $datemsg=$date;
  } elsif ($type eq 'opendate') {
    $status='CLOSED';
    $datemsg = "will open on $date";
  } elsif ($type eq 'duedate') {
    $status='CAN_ANSWER';
    $datemsg = "is due at $date";
  } elsif ($type eq 'answerdate') {
    $status='CLOSED';
    $datemsg = "was due on $lastdate, and answers will be available on $date";
  }
  if ($status eq 'CAN_ANSWER') {
    #check #tries
    my $tries = $Apache::lonhomework::history{"resource.$id.tries"};
    my $maxtries = &Apache::lonnet::EXT("resource.$id.maxtries");
    if ( $tries eq '' ) { $tries = '0'; }
    if ( $maxtries eq '' ) { $maxtries = '2'; } 
    if ($tries >= $maxtries) { $status = 'CANNOT_ANSWER'; } 
  }

  if (($status ne 'CLOSED') && ($Apache::lonhomework::type eq 'exam') &&
      (!$Apache::lonhomework::history{"resource.0.outtoken"})) {
      return ('UNCHECKEDOUT','needs to be checked out');
  }


  &Apache::lonxml::debug("sending back :$status:$datemsg:");
  if (($Apache::lonhomework::browse eq 'F') && ($status eq 'CLOSED')) {
    &Apache::lonxml::debug("should be allowed to browse a resource when closed");
    $status='CAN_ANSWER';
    $datemsg='is closed but you are allowed to view it';
  }
  if ($ENV{'request.state'} eq "construct") {
    &Apache::lonxml::debug("in construction ignoring dates");
    $status='CAN_ANSWER';
    $datemsg='is in under construction';
  }
  return ($status,$datemsg);
}

sub showhash {
  my (%hash) = @_;
  &showhashsubset(\%hash,'');
  return '';
}

sub showhashsubset {
  my ($hash,$keyre) = @_;
  my $resultkey;
  foreach $resultkey (sort keys %$hash) {
    if ($resultkey =~ /$keyre/) {
      if (ref($$hash{$resultkey})) {
	if ($$hash{$resultkey} =~ /ARRAY/ ) {
	  my $string="$resultkey ---- (";
	  foreach my $elm (@{ $$hash{$resultkey} }) {
	    $string.="$elm,";
	  }
	  chop($string);
	  &Apache::lonxml::debug("$string)");
	} else {
	  &Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
	}
      } else {
	&Apache::lonxml::debug("$resultkey ---- $$hash{$resultkey}");
      }
    }
  }
  &Apache::lonxml::debug("\n<br />restored values^</br>\n");
  return '';
}

sub setuppermissions {
  $Apache::lonhomework::browse= &Apache::lonnet::allowed('bre',$ENV{'request.filename'});
  $Apache::lonhomework::viewgrades=&Apache::lonnet::allowed('vgr',$ENV{'request.course.id'});
  return ''
}

sub setupheader {
  my $request=$_[0];
  if ($ENV{'browser.mathml'}) {
    $request->content_type('text/xml');
  } else {
    $request->content_type('text/html');
  }
  if (!$Apache::lonxml::debug && ($ENV{'REQUEST_METHOD'} eq 'GET')) {
    &Apache::loncommon::no_cache($request);
  }
  $request->send_http_header;
  return OK if $request->header_only;
  return ''
}

sub handle_save_or_undo {
  my ($request,$problem,$result) = @_;
  my $file    = &Apache::lonnet::filelocation("",$request->uri);
  my $filebak =$file.".bak";
  my $filetmp =$file.".tmp";
  my $error=0;

  if ($ENV{'form.Undo'} eq 'undo') {
    my $error=0;
    if (!copy($file,$filetmp)) { $error=1; }
    if ((!$error) && (!copy($filebak,$file))) { $error=1; }
    if ((!$error) && (!move($filetmp,$filebak))) { $error=1; }
    if (!$error) {
      $request->print("<p><b>Undid changes, Switched $filebak and $file</b></p>");
    } else {
      $request->print("<p><font color=\"red\" size=\"+1\"><b>Unable to undo, unable to switch $filebak and $file</b></font></p>");
      $error=1;
    }
  } else {
    my $fs=Apache::File->new(">$filebak");
    if (defined($fs)) {
      print $fs $$problem;
      $request->print("<b>Making Backup to $filebak</b><br />");
    } else {
      $request->print("<font color=\"red\" size=\"+1\"><b>Unable to make backup $filebak</b></font>");
      $error=2;
    }
    my $fh=Apache::File->new(">$file");
    if (defined($fh)) {
      print $fh $$result;
      $request->print("<b>Saving Modifications to $file</b><br />");
    } else {
      $request->print("<font color=\"red\" size=\"+1\"><b>Unable to write to $file</b></font>");
      $error|=4;
    }
  }
  return $error;
}

sub analyze {
  my ($request,$file) = @_;
  &Apache::lonxml::debug("Analyze");
  my $result=&Apache::lonnet::ssi($request->uri,('grade_target' => 'analyze'));
  &Apache::lonxml::debug(":$result:");
  (my $garbage,$result)=split(/_HASH_REF__/,$result,2);
  &showhash(&Apache::lonnet::str2hash($result));
  return $result;
}

sub editxmlmode {
  my ($request,$file) = @_;
  my $result;
  my $problem=&Apache::lonnet::getfile($file);
  if ($problem == -1) {
    &Apache::lonxml::error("<b> Unable to find <i>$file</i></b>");
    $problem='';
  }
  if (defined($ENV{'form.editxmltext'}) || defined($ENV{'form.Undo'})) {
    my $error=&handle_save_or_undo($request,\$problem,
				   \$ENV{'form.editxmltext'});
    if (!$error) { $problem=&Apache::lonnet::getfile($file); }
  }
  &Apache::lonhomework::showhashsubset(\%ENV,'^form');
  if ( $ENV{'form.submit'} eq 'Submit Changes and View' ) {
    &Apache::lonhomework::showhashsubset(\%ENV,'^form');
    $ENV{'form.problemmode'}='View';
    &renderpage($request,$file);
  } else {
    my ($rows,$cols) = &Apache::edit::textarea_sizes(\$problem);
    if ($cols > 80) { $cols = 80; }
    $result.='<html><body bgcolor="#FFFFFF">
            <form name="lonhomework" method="POST" action="'.
	      $ENV{'request.uri'}.'">
            <input type="hidden" name="problemmode" value="EditXML" />
            <input type="submit" name="problemmode" value="Discard Edits and View" />
            <input type="submit" name="problemmode" value="Edit" />
            <hr />
            <input type="submit" name="submit" value="Submit Changes" />
            <input type="submit" name="submit" value="Submit Changes and View" />
            <input type="submit" name="Undo" value="undo" />
            <hr />
            <textarea rows="'.$rows.'" cols="'.$cols.'" name="editxmltext">'.
	      &HTML::Entities::encode($problem).'</textarea>
            </form></body></html>';
    $request->print($result);
  }
  return '';
}

sub renderpage {
  my ($request,$file) = @_;

  my (@targets) = &get_target();
  &Apache::lonxml::debug("Running targets ".join(':',@targets));
  foreach my $target (@targets) {
    #my $t0 = [&gettimeofday()];
    my $problem=&Apache::lonnet::getfile($file);
    if ($problem == -1) {
      &Apache::lonxml::error("<b> Unable to find <i>$file</i></b>");
      $problem='';
    }

    my %mystyle;
    my $result = '';
    &Apache::inputtags::initialize_inputtags;
    &Apache::edit::initialize_edit;
    if ($target eq 'analyze') { %Apache::lonhomework::anaylze=(); }
    if ($target eq 'web') {
      my ($symb)=&Apache::lonxml::whichuser();
      if ($symb eq '') {
	if ($ENV{'request.state'} eq "construct") {
	} else {
	  $request->print("Browsing or <a href=\"/adm/ambiguous\">ambiguous</a> reference, submissions ignored<br />");
	}
      }
      #if ($Apache::lonhomework::viewgrades eq 'F') {&createmenu('grade',$request); }
    }
    #if ($target eq 'grade') { &showhash(%Apache::lonhomework::history); }
    #if ($target eq 'web') { &showhash(%ENV); }

    my $default=&Apache::lonnet::getfile('/home/httpd/html/res/adm/includes/default_homework.lcpm');
    if ($default == -1) {
      &Apache::lonxml::error("<b>Unable to find <i>default_homework.lcpm</i></b>");
      $default='';
    }
    &Apache::lonxml::debug("Should be parsing now");
    $result = &Apache::lonxml::xmlparse($request, $target, $problem,
			$default.&setup_vars($target),%mystyle);

    #$request->print("Result follows:");
    if ($target eq 'modified') {
      &handle_save_or_undo($request,\$problem,\$result);
    } else {
      if ($target eq 'analyze') {
	$result=&Apache::lonnet::hashref2str(\%Apache::lonhomework::analyze);
	undef(%Apache::lonhomework::analyze);
      }
      #my $td=&tv_interval($t0);
      #if ( $Apache::lonxml::debug) {
	#$result =~ s:</body>::;
	#$result.="<br />Spent $td seconds processing target $target\n</body>";
      #}
      $request->print($result);
    }
    #$request->print(":Result ends");
    #my $td=&tv_interval($t0);
  }
}

# with no arg it returns a HTML <option> list of the template titles
# with one arg it returns the filename associated with the arg passed
sub get_template_list {
  my ($namewanted,$extension) = @_;
  my $result;
  &Apache::lonxml::debug("Looking for :$extension:");
  foreach my $file (</home/httpd/html/res/adm/includes/templates/*.$extension>) {
    my $name=&Apache::lonnet::metadata($file,'title');
    if ($namewanted && ($name eq $namewanted)) {
      $result=$file;
      last;
    } else {
      $result.="<option>$name</option>";
    }
  }
  return $result;
}

sub newproblem {
    my ($request) = @_;
    my $extension=$request->uri;
    $extension=~s:^.*\.([\w]+)$:$1:;
    &Apache::lonxml::debug("Looking for :$extension:");
    if ($ENV{'form.template'}) {
	use File::Copy;
	my $file = &get_template_list($ENV{'form.template'},$extension);
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	copy($file,$dest);
	&renderpage($request,$dest);
    } elsif($ENV{'form.newfile'}) {
	# I don't like hard-coded filenames but for now, this will work.
	use File::Copy;
	my $templatefilename = 
	    $request->dir_config('lonIncludes').'/templates/blank.problem';
	&Apache::lonxml::debug("$templatefilename");
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	copy($templatefilename,$dest);
	&renderpage($request,$dest);
    }else {
	my $templatelist=&get_template_list('',$extension);
	my $url=$request->uri;
	my $dest = &Apache::lonnet::filelocation("",$request->uri);
	if (!defined($templatelist)) {
	    # We didn't find a template, so just create a blank problem.
	    $request->print(<<ENDNEWPROBLEM);
<body bgcolor="#FFFFFF">
The requested file $url doesn\'t exist. You can create a new $extension <br />
<form action="$url" method="POST">
<input type="submit" name="newfile" value="New $extension"><br />
</form>
</body>
ENDNEWPROBLEM
            return '';
        }
	$request->print(<<ENDNEWPROBLEM);
<body bgcolor="#FFFFFF">
The requested file $url doesn\'t exist. You can create a new $extension <br />
<form action="$url" method="POST">
<input type="submit" value="New $extension"><br />
<select name="template">
$templatelist
</select>
</form>
</body>
ENDNEWPROBLEM
    }
    return '';
}

sub view_or_edit_menu {
  my ($request) = @_;
  my $url=$request->uri;
  $request->print(<<EDITMENU);
<body bgcolor="#FFFFFF">
<form action="$url" method="POST">
Would you like to <input type="submit" name="problemmode" value="View"> or
<input type="submit" name="problemmode" value="Edit"> the problem.
</form>
</body>
EDITMENU
}

sub handler {
  #my $t0 = [&gettimeofday()];
  my $request=$_[0];

  if ( $ENV{'user.name'} eq 'albertel' ) {$Apache::lonxml::debug=1;}

  if (&setupheader($request)) { return OK; }
  $ENV{'request.uri'}=$request->uri;

  #setup permissions
  $Apache::lonhomework::browse= &Apache::lonnet::allowed('bre',$ENV{'request.filename'});
  $Apache::lonhomework::viewgrades=&Apache::lonnet::allowed('vgr',$ENV{'request.course.id'});
  &Apache::lonxml::debug("Permissions:$Apache::lonhomework::browse:$Apache::lonhomework::viewgrades:");
  # some times multiple problemmodes are submitted, need to select
  # the last one
  &Apache::lonxml::debug("Problem Mode ".$ENV{'form.problemmode'});
  if ( defined($ENV{'form.problemmode'}) &&
       ref($ENV{'form.problemmode'}) ) {
    &Apache::lonxml::debug("Problem Mode ".join(",",@$ENV{'form.problemmode'}));
    my $mode=$ENV{'form.problemmode'}->[-1];
    undef $ENV{'form.problemmode'};
    $ENV{'form.problemmode'}=$mode;
  }
  &Apache::lonxml::debug("Problem Mode ".$ENV{'form.problemmode'});
  my $file=&Apache::lonnet::filelocation("",$request->uri);

  #check if we know where we are
  if ($ENV{'request.course.fn'} && !&Apache::lonnet::symbread()) { 
    # if we are browsing we might not be able to know where we are
    if ($Apache::lonhomework::browse ne 'F') {
      #should know where we are, so ask
      $request->internal_redirect('/adm/ambiguous'); return;
    }
  }

  if ($ENV{'request.state'} eq "construct") {
    if ($ENV{'form.resetdata'} eq 'Reset Submissions') {
      my ($symb,$courseid,$domain,$name) = &Apache::lonxml::whichuser();
      &Apache::lonnet::tmpreset($symb,'',$domain,$name);
    }
    if ( -e $file ) {
      if (!(defined $ENV{'form.problemmode'})) {
	#first visit to problem in construction space
	#&view_or_edit_menu($request);
	$ENV{'form.problemmode'}='View';
	&renderpage($request,$file);
      } elsif ($ENV{'form.problemmode'} eq 'EditXML') {
	&editxmlmode($request,$file);
      } elsif ($ENV{'form.problemmode'} eq 'Answer Distribution') {
	&analyze($request,$file);
      } else {
	&renderpage($request,$file);
      }
    } else {
      # requested file doesn't exist in contruction space
      &newproblem($request);
    }
  } else {
    # just render the page normally outside of construction space
    &Apache::lonxml::debug("not construct");
    &renderpage($request,$file);
  }
  #my $td=&tv_interval($t0);
  #&Apache::lonxml::debug("Spent $td seconds processing");
  # &Apache::lonhomework::send_footer($request);
  # always turn off debug messages
  $Apache::lonxml::debug=0;
  return OK;

}

1;
__END__
