# The LearningOnline Network with CAPA
# option list style responses
#
# $Id: optionresponse.pm,v 1.41 2002/06/21 20:36:30 albertel Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 2/21 Guy
# 01/18 Alex
package Apache::optionresponse;
use strict;
use Apache::response;

BEGIN {
  &Apache::lonxml::register('Apache::optionresponse',('optionresponse'));
}

sub start_optionresponse {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result='';
  #when in a option response use these
  &Apache::lonxml::register('Apache::optionresponse',('foilgroup','foil','conceptgroup'));
  push (@Apache::lonxml::namespace,'optionresponse');
  my $id = &Apache::response::start_response($parstack,$safeeval);
  if ($target eq 'edit') {
    $result.=&Apache::edit::start_table($token).
	"<tr><td>Multiple Option Response Question</td><td>Delete:".
	&Apache::edit::deletelist($target,$token)
	."</td><td>&nbsp;".
        &Apache::edit::end_row().
        &Apache::edit::start_spanning_row().
        "\n";
    $result.=&Apache::edit::text_arg('Max Number Of Shown Foils:','max',$token,'4').
        &Apache::edit::end_row().
	&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,
						 $safeeval,'max');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  } elsif ($target eq 'meta') {
    $result=&Apache::response::meta_package_write('optionresponse');
  } elsif ($target eq 'analyze') {
    my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
    push (@{ $Apache::lonhomework::analyze{"parts"} },$part_id);
  }
#  if ($target eq 'tex') {$result .= '\begin{enumerate}';}
  return $result;
}

sub end_optionresponse {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  &Apache::response::end_response;
  pop @Apache::lonxml::namespace;
  &Apache::lonxml::deregister('Apache::optionresponse',('foilgroup','foil','conceptgroup'));
  my $result;
  if ($target eq 'edit') { $result=&Apache::edit::end_table(); }
#  if ($target eq 'tex') {$result .= '\end{enumerate}';}
  return $result;
}

%Apache::response::foilgroup={};
sub start_foilgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;

  my $result='';
  %Apache::response::foilgroup={};
  $Apache::optionresponse::conceptgroup=0;
  &Apache::response::setrandomnumber();
  if ($target eq 'edit') {
    my $optionlist="<option></option>\n";
    my $option;
    my @opt;
    eval '@opt ='. &Apache::lonxml::get_param('options',$parstack,$safeeval);
    my $count=1;
    foreach $option (@opt) {
      $optionlist.="<option value=\"$count\">$option</option>\n";
      $count++;
    }
    my $insertlist=&Apache::edit::insertlist($target,$token);
    $result.=&Apache::edit::start_table($token);
    $result.= (<<ENDTABLE);
      <tr><td>Select Options</td>
        <td>
	  Add new Option: <input type="text" name="$Apache::lonxml::curdepth.options" />
        </td>
        <td>Delete an Option:
	  <select name="$Apache::lonxml::curdepth.deleteopt">$optionlist</select>
ENDTABLE
    $result.= &Apache::edit::end_row();
    $result.= &Apache::edit::start_spanning_row();
    $result.= $insertlist.'<br />';
  }
  if ($target eq 'modified') {
    my @options;
    my $optchanged=0;
    eval '@options ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
    if ($ENV{"form.$Apache::lonxml::curdepth.deleteopt"}) {
      my $delopt=$ENV{"form.$Apache::lonxml::curdepth.deleteopt"};
      &Apache::lonxml::debug("Deleting :$delopt:");
      splice(@options,$delopt-1,1);
      $optchanged=1;
    }
    if ($ENV{"form.$Apache::lonxml::curdepth.options"}) {
      my $newopt = $ENV{"form.$Apache::lonxml::curdepth.options"};
      if ($options[0]) {
	push(@options,$newopt);
      } else {
	$options[0]=$newopt;
      }
      $optchanged=1;
    }
    if ($optchanged) {
      $result = "<foilgroup options=\"(";
      foreach my $option (@options) {
	$option=~s/\'/\\\'/g;
	&Apache::lonxml::debug("adding option :$option:");
	$result .="'".$option."',";
      }
      chop $result;
      $result.=')">';
    } # else nothing changed so just use the default mechanism
  }
  if ($target eq 'tex') {$result .= '\begin{enumerate}';}
  return $result;
}

sub end_foilgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  
  my $result;
  if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' || $target eq 'tex' || $target eq 'analyze') {
    my $name;
    my ($count,$max) = &getfoilcounts($parstack,$safeeval);
    if ($count>$max) { $count=$max } 
    &Apache::lonxml::debug("Count is $count from $max");
    my @opt;
    eval '@opt ='.&Apache::lonxml::get_param('options',$parstack,$safeeval);
    &Apache::lonxml::debug("Options are $#opt");
    if ($target eq 'web' || $target eq 'tex') {
      $result.=&displayfoils($target,$count,@opt);
    } elsif ( $target eq 'answer') {
      $result.=&displayanswers($count,@opt);
    } elsif ( $target eq 'analyze') {
      my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
      foreach my $name (@{ $Apache::response::foilgroup{'names'} }) {
	if (defined($Apache::lonhomework::analyze{"$part_id.foil.value.$name"})) { next; }
	push (@{ $Apache::lonhomework::analyze{"$part_id.foils"} },$name);
	$Apache::lonhomework::analyze{"$part_id.foil.value.$name"} =
	  $Apache::response::foilgroup{"$name.value"};
	$Apache::lonhomework::analyze{"$part_id.foil.text.$name"} =
	  $Apache::response::foilgroup{"$name.text"};
      }
      push (@{ $Apache::lonhomework::analyze{"$part_id.options"} },@opt);
      push (@{ $Apache::lonhomework::analyze{"$part_id.shown"} },&whichfoils($count));
    } elsif ( $target eq 'grade') {
      if ( defined $ENV{'form.submitted'}) {
	my @whichopt = &whichfoils($count);
	my $temp=1;my $name;
	my %responsehash;
	my $right=0;
	my $wrong=0;
	my $ignored=0;
	foreach $name (@whichopt) {
	  my $response = $ENV{'form.HWVAL_'.$Apache::inputtags::response['-1'].":$temp"};
	  $responsehash{$name}=$response;
	  if ( $response =~ /[^\s]/) {
	    my $value=$Apache::response::foilgroup{$name.'.value'};
	    &Apache::lonxml::debug("submitted a $response for $value<br />\n");
	    if ($value eq $response) {$right++;} else {$wrong++;}
	  } else {
	    $ignored++;
	  }
	  $temp++;
	}
	my $part=$Apache::inputtags::part;
	my $id = $Apache::inputtags::response['-1'];
	my $responsestr=&Apache::lonnet::hash2str(%responsehash);
	my %previous=&Apache::response::check_for_previous($responsestr,
							   $part,$id);
	&Apache::lonxml::debug("Got $right right and $wrong wrong, and $ignored were ignored");
	my $ad;
	if ($wrong==0 && $ignored==0) {
	  $ad='EXACT_ANS';
	} elsif ($wrong==0 && $right==0) {
	  #nothing submitted
	} else {
	  if ($ignored==0) {
	    $ad='INCORRECT';
	  } else {
	    $ad='MISSING_ANSWER';
	  }
	}
	$Apache::lonhomework::results{"resource.$part.$id.submission"}=
	  $responsestr;
	$Apache::lonhomework::results{"resource.$part.$id.awarddetail"}=
	  $ad;
	&Apache::response::handle_previous(\%previous,$ad);
      }
    }
  } elsif ($target eq 'edit') {
    $result.=&Apache::edit::end_table();
  }  
  if ($target eq 'tex') {$result .= '\end{enumerate}';}
  return $result;
}

sub getfoilcounts {
  my ($parstack,$safeeval)=@_;
  my $max = &Apache::lonxml::get_param('max',$parstack,$safeeval,'-2');
  # +1 since instructors will count from 1
  my $count = $#{ $Apache::response::foilgroup{'names'} }+1;
  if (&Apache::response::showallfoils()) { $max=$count; }
  return ($count,$max);
}

sub whichfoils {
  my ($max)=@_;
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @whichopt =();
  while ((($#whichopt+1) < $max) && ($#names > -1)) {
    &Apache::lonxml::debug("Have $#whichopt max is $max");
    my $aopt;
    if (&Apache::response::showallfoils()) {
      $aopt=0;
    } else {
      $aopt=int(rand($#names+1));
    }
    &Apache::lonxml::debug("From $#whichopt $max $#names elms, picking $aopt");
    $aopt=splice(@names,$aopt,1);
    &Apache::lonxml::debug("Picked $aopt");
    push (@whichopt,$aopt);
  }
  return @whichopt;
}

sub displayanswers {
  my ($max,@opt)=@_;
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @whichopt = &whichfoils($max);
  my $result=&Apache::response::answer_header('optionresponse');
  foreach my $name (@whichopt) {
    $result.=&Apache::response::answer_part('optionresponse',
		     $Apache::response::foilgroup{$name.'.value'})
  }
  $result.=&Apache::response::answer_footer('optionresponse');
  return $result;
}

sub displayfoils {
  my ($target,$max,@opt)=@_;
  my @names = @{ $Apache::response::foilgroup{'names'} };
  my @truelist;
  my @falselist;
  my $result;
  my $name;
  $Apache::optionresponse::displayoptionintex=0;
  my @whichopt = &whichfoils($max);
  my $part=$Apache::inputtags::part;
  my $id=$Apache::inputtags::response[-1];
  if (($Apache::lonhomework::history{"resource.$part.solved"} =~ /^correct/)  || ($Apache::inputtags::status[-1] eq  'SHOW_ANSWER')) {
    foreach $name (@whichopt) {
      if ($target eq 'web') {
	  $result.="<br />";
      } elsif ($target eq 'tex') {
	$result.='\newline';
      }
      $result .=$Apache::response::foilgroup{$name.'.value'}.
	  ":".$Apache::response::foilgroup{$name.'.text'}."\n";
    }
  } else {
    my $temp=1;
    my %lastresponse=&Apache::lonnet::str2hash($Apache::lonhomework::history{"resource.$part.$id.submission"});
    foreach $name (@whichopt) {
      my $lastopt=$lastresponse{$name};
      my $optionlist="<option></option>\n";
      my $option;
      foreach $option (@opt) {
	if ($option eq $lastopt) {
	  $optionlist.="<option selected=\"on\">$option</option>\n";
	} else {
	  $optionlist.="<option>$option</option>\n";
	}
      }
      if ($target ne 'tex') {
	  $result.="<br /><select name=\"HWVAL_$Apache::inputtags::response['-1']:$temp\">"
	      .$optionlist
		  ."</select>\n".$Apache::response::foilgroup{$name.'.text'}."\n";
	  $temp++;
      } else {
	  if ($Apache::optionresponse::displayoptionintex == 0) {
              my $durtyoptionlist = $optionlist;
	      $durtyoptionlist =~ s/<option><\/option>/\\item \[\] The possible answers are:/;
	      $durtyoptionlist =~ s/<option>/{\\bf /g;
	      $durtyoptionlist =~ s/<\/option>/},/g;
	      $durtyoptionlist =~ s/,$/\./g;
	      $durtyoptionlist =~ s/>/\$>\$/g;
	      $durtyoptionlist =~ s/</\$<\$/g;
	      $durtyoptionlist =~ s/=/\$=\$/g;
	      $result.= $durtyoptionlist.$Apache::response::foilgroup{$name.'.text'};
	      $Apache::optionresponse::displayoptionintex=1;
	  } else {
	      $result.= $Apache::response::foilgroup{$name.'.text'};
	  }
      } 
    }
  } 
  if ($target ne 'tex') {
      return $result."<br />";
  }
  else {
      return $result;
  }
}


sub start_conceptgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  $Apache::optionresponse::conceptgroup=1;
  %Apache::response::conceptgroup={};
  my $result;
  if ($target eq 'edit') {
    $result.=&Apache::edit::tag_start($target,$token,"Concept Grouped Foils");
    $result.=&Apache::edit::text_arg('Concept:','concept',$token,'50').
        &Apache::edit::end_row().&Apache::edit::start_spanning_row();
  }
  if ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,'concept');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  }
  return $result;
}

sub end_conceptgroup {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  $Apache::optionresponse::conceptgroup=0;
  my $result='';
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer'
      || $target eq 'tex' || $target eq 'analyze') {
    #if not there aren't any foils to display and thus no question
    if (defined(@{ $Apache::response::conceptgroup{'names'} })) {
      my @names = @{ $Apache::response::conceptgroup{'names'} };
      my $pick=int rand $#names+1;
      my $name=$names[$pick];
      push @{ $Apache::response::foilgroup{'names'} }, $name;
      $Apache::response::foilgroup{"$name.value"} =
	$Apache::response::conceptgroup{"$name.value"};
      $Apache::response::foilgroup{"$name.text"} =
	$Apache::response::conceptgroup{"$name.text"};
      my $concept = &Apache::lonxml::get_param('concept',$parstack,$safeeval);
      $Apache::response::foilgroup{"$name.concept"} = $concept;
      &Apache::lonxml::debug("Selecting $name in $concept");
      if ($target eq 'analyze') {
	my $part_id="$Apache::inputtags::part.$Apache::inputtags::response[-1]";
        push (@{ $Apache::lonhomework::analyze{"$part_id.concepts"} },
	      $concept);
	$Apache::lonhomework::analyze{"$part_id.concept.$concept"}=
	  $Apache::response::conceptgroup{'names'};
	foreach my $name (@{ $Apache::response::conceptgroup{'names'} }) {
	  push (@{ $Apache::lonhomework::analyze{"$part_id.foils"} },
		$name);
	  $Apache::lonhomework::analyze{"$part_id.foil.value.$name"} =
	    $Apache::response::conceptgroup{"$name.value"};
	  $Apache::lonhomework::analyze{"$part_id.foil.text.$name"} =
	    $Apache::response::conceptgroup{"$name.text"};
	}
      }
    }
  } elsif ($target eq 'edit') {
    $result=&Apache::edit::end_table();
  }
  return $result;
}

sub insert_conceptgroup {
  my $result="\n\t\t<conceptgroup concept=\"\">".&insert_foil()."\n\t\t</conceptgroup>\n";
  return $result;
}

sub start_foil {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $result='';
  if ($target eq 'web' || $target eq 'tex' || $target eq 'analyze' ) {
    &Apache::lonxml::startredirection;
  } elsif ($target eq 'edit') {
    $result=&Apache::edit::tag_start($target,$token,"Foil");
    my $level='-2';
    if ($$tagstack['-2'] eq 'conceptgroup') { $level = '-3'; }
    my @opt;
    eval '@opt ='.&Apache::lonxml::get_param('options',$parstack,$safeeval,$level);
    $result.=&Apache::edit::text_arg('Name:','name',$token);
    $result.= &Apache::edit::select_or_text_arg('Correct Option:','value',['unused',(@opt)],$token,'15');
    $result .=&Apache::edit::end_row().&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified') {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,'value','name');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  } 
  if ($target eq 'tex') {$result .= '\item ';}
  return $result;
}

sub end_foil {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval,$style)=@_;
  my $text ='';
  my $result = '';
  if ($target eq 'web' || $target eq 'tex') { 
      $text=&Apache::lonxml::endredirection; 
  }
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' 
      || $target eq 'tex' || $target eq 'analyze') {
    my $value = &Apache::lonxml::get_param('value',$parstack,$safeeval);
    if ($value ne 'unused') {
      my $name = &Apache::lonxml::get_param('name',$parstack,$safeeval);
      &Apache::lonxml::debug("Got a name of :$name:");
      if (!$name) { $name=$Apache::lonxml::curdepth; }
      &Apache::lonxml::debug("Using a name of :$name:");
      if ( $Apache::optionresponse::conceptgroup
	   && !&Apache::response::showallfoils() ) {
	push @{ $Apache::response::conceptgroup{'names'} }, $name;
	$Apache::response::conceptgroup{"$name.value"} = $value;
	$Apache::response::conceptgroup{"$name.text"} = $text;	
      } else {
	push @{ $Apache::response::foilgroup{'names'} }, $name;
	$Apache::response::foilgroup{"$name.value"} = $value;
	$Apache::response::foilgroup{"$name.text"} = $text;
      }
    }
  }
  if ($target eq 'edit') {
    $result.= &Apache::edit::tag_end($target,$token,'');
  }
  return $result;
}

sub insert_foil {
  return '
<foil name="" value="unused">
<startouttext />
<endouttext />
</foil>';
}
1;
__END__
 
