# The LearningOnline Network with CAPA 
# definition of tags that give a structure to a document
#
# $Id: structuretags.pm,v 1.100 2002/06/27 18:47:23 sakharuk Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# 2/19 Guy
# 6/26/2001 fixed extra web display at end of <web></web> tags
# 8/17,8/18,8/20 Gerd Kortemeyer

package Apache::structuretags; 

use strict;
use Apache::lonnet;

BEGIN {
  &Apache::lonxml::register('Apache::structuretags',('block','while','randomlist','problem','library','web','tex','part','preduedate','postanswerdate','solved','notsolved','startouttext','endouttext'));
#  &Apache::lonxml::register_insert('problem','',('part','postanswerdate','preduedate'))
}

sub start_web {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $bodytext=&Apache::lonxml::get_all_text("/web",$$parser[$#$parser]);
  if ($target eq 'web') {
    return $bodytext;
  } 
  return '';
}

sub end_web {
    return '';
}

sub start_tex {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $bodytext=&Apache::lonxml::get_all_text("/tex",$$parser[$#$parser]);
  if ($target eq 'tex') {
    return $bodytext
  }
  return '';
}

sub end_tex {
    return '';
}

sub page_start {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my %found;
  foreach my $taginside ($tagstack) {
    foreach my $taglookedfor ('html','body','form') {
      if ($taginside =~ /^$taglookedfor$/i) { $found{$taglookedfor} = 1; }
    }
  }

  my $result;
  my $head_tag_start;
  if (!defined($found{'html'})) {
    $result=&Apache::londefdef::start_html($target,$token,$tagstack,$parstack,
					   $parser,$safeeval);
    $head_tag_start='<head>'.&Apache::lonxml::registerurl(undef,$target);
  }
  my $body_tag_start;
  if (!defined($found{'body'})) {
    $body_tag_start='<body onLoad="'.&Apache::lonxml::loadevents().'" '.
      'onUnload="'.&Apache::lonxml::unloadevents().'" ';
    my $background=&Apache::lonxml::get_param('background',$parstack,$safeeval);
    if ($background) {
      $Apache::lonxml::extlinks[$#Apache::lonxml::extlinks+1]=
	$background;
      $body_tag_start.='background="'.$background.'" ';
    } else {
      my $bgcolor=&Apache::lonxml::get_param('bgcolor',$parstack,$safeeval);
      if ($bgcolor) {
	$body_tag_start.='bgcolor="'.$bgcolor.'" ';
      } else {
	$body_tag_start.='bgcolor="#ffffff"';
      }
    }
    $body_tag_start.='>';
  }
  return ($result,$head_tag_start,$body_tag_start);
}

sub initialize_storage {
  %Apache::lonhomework::results=();
  my ($symb,$courseid,$domain,$name) = &Apache::lonxml::whichuser();
  if ($ENV{'request.state'} eq 'construct') {
    %Apache::lonhomework::history=
      &Apache::lonnet::tmprestore($ENV{'request.uri'},'',$domain,$name);
    my ($temp)=keys %Apache::lonhomework::history ;
    &Apache::lonxml::debug("Return message of $temp");
  } else {
    %Apache::lonhomework::history=
      &Apache::lonnet::restore($symb,$courseid,$domain,$name);
  }
  #ignore error conditions
  my ($temp)=keys %Apache::lonhomework::history ;
  if ($temp =~ m/^error:.*/) { %Apache::lonhomework::history=(); }
}

# -------------------------------------------------------------finalize_storage
# Stores away the result has to a student's environment
# checks form.grade_ for specific values, other wises stores
# to the running users environment
sub finalize_storage {
  my $result;
  my ($temp) = keys %Apache::lonhomework::results;
  if ( $temp ne '' ) {
    my ($symb,$courseid,$domain,$name) = &Apache::lonxml::whichuser();
    if ($ENV{'request.state'} eq 'construct') {
      $result=&Apache::lonnet::tmpstore(\%Apache::lonhomework::results,
				      $ENV{'request.uri'},'',$domain,$name);
      &Apache::lonxml::debug('Construct Store return message:'.$result);
    } else {
      $result=&Apache::lonnet::cstore(\%Apache::lonhomework::results,
				      $symb,$courseid,$domain,$name);
      &Apache::lonxml::debug('Store return message:'.$result);
    }
  }
  return $result;
}

sub checkout_msg {
return (<<ENDCHECKOUT);
<h2>The resource needs to be checked out</h2>
As a resource gets checked out, a unique timestamped ID is given to it, and a
permanent record is left in the system.<p />
<font color=red>
Checking out resources is subject to course policies, and may exclude future
credit even if done erroneously.<p />
</font>
<form name="checkout" method="POST" action="$ENV{'request.uri'}">
<input type="hidden" name="doescheckout" value="yes" />
<input type="button" name="checkoutbutton" value="Check out Exam for Viewing" onClick="javascript:if (confirm('Check out Exam?')) { document.checkout.submit(); }" />
</form>
ENDCHECKOUT
}

sub start_problem {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;

  # meta is called from lonpublisher, which doesn't uses the normal
  # lonhomework method of parsing the file which means that inputtags 
  # won't get reset
  if ( $Apache::inputtags::part ne '' && $target != 'meta' ) {
    &Apache::lonxml::error('Only one problem allowed in a .problem file');
    my $bodytext=&Apache::lonxml::get_all_text("/problem",$$parser[-1]);
    return '';
  }
#intialize globals
  $Apache::inputtags::part='0';
  @Apache::inputtags::responselist = ();
  @Apache::inputtags::previous=();
  if ($target ne 'analyze') {
    &initialize_storage();
    if ($target eq 'web') {
      &Apache::lonhomework::showhash(%Apache::lonhomework::history);
    }
    $Apache::lonhomework::type=&Apache::lonnet::EXT('resource.0.type');
    &Apache::lonxml::debug("Found this to be of type :$Apache::lonhomework::type:");
  }
  if ($Apache::lonhomework::type eq '') {
    my $uri=$ENV{'request.uri'};
    if ($uri=~/\.(\w+)$/) {
      $Apache::lonhomework::type=$1;
      &Apache::lonxml::debug("Using type of $1");
    } else {
      $Apache::lonhomework::type='problem';
      &Apache::lonxml::debug("Using default type, problem, :$uri:");
    }
  }

  #added vars to the scripting enviroment
  my $expression='$external::part='.$Apache::inputtags::part.';';
  &Apache::run::run($expression,$safeeval);
  my $status;
  my $accessmsg;

  #should get back a <html> or the neccesary stuff to start XML/MathML
  my ($result,$head_tag_start,$body_tag_start)=
    &page_start($target,$token,$tagstack,$parstack,$parser,$safeeval);
  if ($target eq 'tex' and $ENV{'request.symb'} =~ m/\.page_/) { $result = '';}

  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer' || $target eq 'tex') {
    #handle exam checkout
    if ($Apache::lonhomework::type eq 'exam') {
      my $token=$Apache::lonhomework::history{"resource.0.outtoken"};
      if (($ENV{'form.doescheckout'}) && (!$token)) {
	$token=&Apache::lonxml::maketoken();
	$Apache::lonhomework::history{"resource.0.outtoken"}=$token;
      }
      $body_tag_start.=&Apache::lonxml::printtokenheader($target,$token);
    }

    #handle rand seed in construction space
    my $rndseed;
    if ($ENV{'request.state'} eq "construct") {
      $rndseed=$ENV{'form.rndseed'};
      if (!$rndseed) {
	$rndseed=time;
	$ENV{'form.rndseed'}=$rndseed;
      }
      &Apache::run::run('$external::randomseed='.$rndseed.';',$safeeval);
    }
    ($status,$accessmsg) = &Apache::lonhomework::check_access('0');
    push (@Apache::inputtags::status,$status);
    my $expression='$external::datestatus="'.$status.'";';
    $expression.='$external::gradestatus="'.$Apache::lonhomework::history{"resource.0.solved"}.'";';
    &Apache::run::run($expression,$safeeval);
    if (( $status eq 'CLOSED' ) ||
        ( $status eq 'UNCHECKEDOUT') ||
        ( $status eq 'BANNED')) {
      my $bodytext=&Apache::lonxml::get_all_text("/problem",$$parser[$#$parser]);
      if ( $target eq "web" ) {
	$result.= $head_tag_start.'</head>';
        my $msg=$body_tag_start.
	    '<h1>Not open to be viewed</h1>';
        if ($status eq 'CLOSED') {
	    $msg.='The problem '.$accessmsg;
	} elsif ($status eq 'UNCHECKEDOUT') {
            $msg.=&checkout_msg;
        }
	$result.=$msg.'<br />';
      }
    } elsif ($target eq 'web') {
      my $name= &Apache::lonxml::get_param('name',$parstack,$safeeval);
      if ($name eq '') { 
	$name=&Apache::lonnet::EXT('resource.title');
	if ($name eq 'con_lost') { $name = ''; }
      }
      $Apache::lonhomework::name=$name;
      if ($status eq 'CAN_ANSWER') {
	# create a page header and exit
	$result.="$head_tag_start<title>$name</title></head>\n
              $body_tag_start\n
              <form name=\"lonhomework\" method=\"POST\" action=\"".$ENV{'request.uri'}."\">".
		'<input type="hidden" name="submitted" value="yes" />';
	if ($ENV{'request.state'} eq "construct") {
	  $result.=
	    '<input type="hidden" name="problemmode" value="View" />
             <input type="submit" name="problemmode" value="Edit" />
             <input type="submit" name="problemmode" value="EditXML" />
             Random Seed:<input type="text" name="rndseed" width="10" value="'.
	       $rndseed.'" />
             <input type="submit" name="changerandseed" value="Change" />
             <input type="submit" name="resetdata" value="Reset Submissions" />
             <input type="checkbox" name="showallfoils" ';
	  if (defined($ENV{'form.showallfoils'})) { $result.='checked="on"'; }
	  $result.= ' /> Show All Foils
             <hr />';
	}
	# if we are viewing someone else preserve that info
	if (defined $ENV{'form.grade_symb'}) {
	  foreach my $field ('symb','courseid','domain','username') {
	    $result .= '<input type="hidden" name="grade_'.$field.
	      '" value="'.$ENV{"form.grade_$field"}.'" />'."\n";
	  }
	}
      } elsif ($status eq 'SHOW_ANSWER' || $status eq 'CANNOT_ANSWER'
	       || $status eq 'CLOSED') {
	$result.=$head_tag_start.
	  "<title>$name</title></head>\n$body_tag_start\n";
      }
    } elsif ($target eq 'tex') {
	my $id = $Apache::inputtags::part;
	my $duedate = &Apache::lonnet::EXT("resource.$id.duedate"); 
	$duedate = POSIX::strftime("%c",localtime($duedate));
	if (not $ENV{'request.symb'} =~ m/\.page_/) {
	    $result .= '\begin{document} \noindent\fbox{Due date: '.$duedate.'} \vskip 1 mm';	  
	} else {
	    $result .= '\parbox{\minipagewidth}{\vskip 1mm\fbox{Due date: '.$duedate.'} \\\\\\\\';
	}
    }
  } elsif ($target eq 'edit') {
    $result.=$head_tag_start."</head>".$body_tag_start.
      '<form name="lonhomework" method="POST" action="'.
	$ENV{'request.uri'}.'">
       <input type="hidden" name="submitted" value="edit" />
       <input type="hidden" name="problemmode" value="Edit" />
       <input type="submit" name="problemmode" value="Discard Edits and View" />
       <input type="submit" name="problemmode" value="EditXML" />
       <input type="submit" name="Undo" value="undo" /> <hr />
       <input type="submit" name="submit" value="Submit Changes" />
       <input type="submit" name="submit" value="Submit Changes and View" /><br />
      ';
    my $temp=&Apache::edit::insertlist($target,$token);
    $result.=$temp;
  } elsif ($target eq 'modified') {
    $result=$token->[4];
    $result.=&Apache::edit::handle_insert();
  } else {
    # page_start returned a starting result, delete it if we don't need it
    $result = '';
  }
  return $result;
}

sub end_problem {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result='';
  my $status=$Apache::inputtags::status['-1'];
  if ($target eq 'grade' || $target eq 'web' || $target eq 'answer' ) {
    if ( $target eq 'grade' && $Apache::inputtags::part eq '0') {
      # if part is zero, no <part>s existed, so we need to the grading
      &Apache::inputtags::grade;
    } elsif ( $target eq 'web' && $Apache::inputtags::part eq '0' && 
	      $status ne 'UNCHECKEDOUT') {
      # if part is zero, no <part>s existed, so we need show the current 
      # grading status
      $result.= &Apache::inputtags::gradestatus($Apache::inputtags::part);
    }
    if (
	($target eq 'web' && ($ENV{'request.state'} ne 'construct')) ||
	($target eq 'answer')
       ) {
      if ($status eq 'CAN_ANSWER') {
	$result.="</form></body>\n"; 
      } elsif ($status eq 'SHOW_ANSWER' || $status eq 'CANNOT_ANSWER' ||
	       $status eq 'UNCHECKEDOUT' ) {
	$result.="</body>\n";
      }
      $result.=&Apache::lonxml::xmlend();
    }
    if ($target eq 'grade') { 
      &Apache::lonhomework::showhash(%Apache::lonhomework::results);
      &finalize_storage();
    }
  } elsif ($target eq 'meta') {
    if ($Apache::inputtags::part eq '0') {
      $result=&Apache::response::mandatory_part_meta;
    }
  } elsif ($target eq 'edit') {
    &Apache::lonxml::debug("in end_problem with $target, edit");
    $result ='<br /><input type="submit" name="submit" value="Submit Changes and Edit" />';
    $result.='<input type="submit" name="submit" value="Submit Changes and View" />';
  } elsif ($target eq 'tex') {
      $result .= '\vskip 0.5mm\noindent\makebox[9.0cm][b]{\hrulefill}';
      if (not $ENV{'request.symb'} =~ m/\.page_/) {
	  $result .= '\end{document} ';
      } else {
	  $result .= '} ';
      }
  }
  return $result;
}

sub start_library {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my ($result,$head_tag_start,$body_tag_start)=
    &page_start($target,$token,$tagstack,$parstack,$parser,$safeeval);
  if ($target eq 'edit') {
    $result.=$head_tag_start."</head>".$body_tag_start.
      '<form name="lonhomework" method="POST" action="'.$ENV{'request.uri'}.'">
       <input type="hidden" name="submitted" value="edit" />
       <input type="hidden" name="problemmode" value="Edit" />
       <input type="submit" name="problemmode" value="View" />
       <input type="submit" name="Undo" value="undo" /> <hr />
      ';
    my $temp=&Apache::edit::insertlist($target,$token);
    $result.=$temp;
    return $result;
  }
  if ($target eq 'modified') {
    $result=$token->[4];
    $result.=&Apache::edit::handle_insert();
    return $result;
  }
  return '';
}

sub end_library {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result='';
  if ($target eq 'edit') {
    $result='<br /><input type="submit" name="submit" value="Submit Changes" />';
  }
  return $result;
}

sub start_block {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;

  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer') {
    my $code = @$parstack[$#$parstack];
    $code =~ s/\"//g;
    $code .=';return $condition;';
    #  print "<br />$code<br />";
    my $result = &Apache::run::run($code,$safeeval);
    &Apache::lonxml::debug("block :$code: returned :$result:");
    if ( ! $result ) { 
      my $skip=&Apache::lonxml::get_all_text("/block",$$parser[$#$parser]);
      &Apache::lonxml::debug("skipping ahead :$skip: $$parser[$#$parser]");
    }
  }
  return "";
}

sub end_block {
  return '';
}

sub start_while {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;

  my $code = @$parstack[$#$parstack];
  $code =~ s/\"//g;
  $code .=';return $condition;';

  push( @Apache::structuretags::whileconds, $code); 
  my $result = &Apache::run::run($code,$safeeval);
  my $bodytext=$$parser[$#$parser]->get_text("/while");
  push( @Apache::structuretags::whilebody, $bodytext);
  if ( $result ) { 
    &Apache::lonxml::newparser($parser,\$bodytext);
  }
  return "";
}

sub end_while {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $code = pop @Apache::structuretags::whileconds;
  my $bodytext = pop @Apache::structuretags::whilebody;
  my $result = &Apache::run::run($code,$safeeval);
  if ( $result ) { 
    &Apache::lonxml::newparser($parser,\$bodytext);
  } 
  return "";
}

# <randomlist> 
#  <tag1>..</tag1>
#  <tag2>..</tag2>
#  <tag3>..</tag3>
#  ... 
# </randomlist>
sub start_randomlist {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result;
  if ($target eq 'answer' || $target eq 'grade' || $target eq 'web') {
    my $body= &Apache::lonxml::get_all_text("/randomlist",$$parser[$#$parser]);
    my $b_parser= HTML::TokeParser->new(\$body);
    my $b_tok;
    my @randomlist;
    my $list_item;
    while($b_tok = $b_parser->get_token() ) {
      if($b_tok->[0] eq 'S') { # start tag
	# get content of the tag until matching end tag
	# get all text upto the matching tag
	# and push the content into @randomlist
	$list_item = &Apache::lonxml::get_all_text('/'.$b_tok->[1],$b_parser);
	$list_item = "$b_tok->[4]"."$list_item"."</$b_tok->[1]>";
	push(@randomlist,$list_item);
	#  print "<br /><b>START-TAG $b_tok->[1], $b_tok->[4], $list_item</b>";
      }
      if($b_tok->[0] eq 'T') { # text
	# what to do with text in between tags?
	#  print "<b>TEXT $b_tok->[1]</b><br />";
      }
      # if($b_tok->[0] eq 'E') { # end tag, should not happen
      #  print "<b>END-TAG $b_tok->[1]</b><br />";
      # }
    }
    my @idx_arr = (0 .. $#randomlist);
    &Apache::structuretags::shuffle(\@idx_arr);
    my $bodytext = '';
    my $show=$#randomlist;
    my $showarg=&Apache::lonxml::get_param('show',$parstack,$safeeval);
    $showarg--;
    if ( ($showarg >= 0) && ($showarg < $show) ) { $show = $showarg; }
    for(0 .. $show) {
      $bodytext .= "$randomlist[ $idx_arr[$_] ]";
    }
    &Apache::lonxml::newparser($parser,\$bodytext);
  } elsif ($target eq 'edit' ) {
    $result .= &Apache::edit::tag_start($target,$token);
    $result .= &Apache::edit::text_arg('Maximum Tags to Show:','show',$token,5);
    $result .= &Apache::edit::end_row().&Apache::edit::start_spanning_row();
  } elsif ($target eq 'modified' ) {
    my $constructtag=&Apache::edit::get_new_args($token,$parstack,$safeeval,
						 'show');
    if ($constructtag) { $result = &Apache::edit::rebuild_tag($token); }
  }
  return $result;
}

sub shuffle {
    my $a=shift;
    my $i;
    if (defined(@$a)) {
      &Apache::response::setrandomnumber();
      for($i=@$a;--$i;) {
	my $j=int rand($i+1);
	next if $i == $j;
	@$a[$i,$j] = @$a[$j,$i];
      }
    }
}

sub end_randomlist {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result;
  if ($target eq 'edit' ) {
    $result=&Apache::edit::tag_end($target,$token,'End Randomly Parsed Block');
  }
  return $result;
}

sub start_part {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result='';
  my $id= &Apache::lonxml::get_param('id',$parstack,$safeeval);
  if ($id eq '') { $id = $Apache::lonxml::curdepth; }
  $Apache::inputtags::part=$id;
  @Apache::inputtags::responselist = ();
  @Apache::inputtags::previous=();
  if ($target eq 'meta') {
    return &Apache::response::mandatory_part_meta;
  } elsif ($target eq 'web' || $target eq 'grade' || $target eq 'answer' || $target eq 'tex') {
    my ($status,$accessmsg) = &Apache::lonhomework::check_access($id);
    push (@Apache::inputtags::status,$status);
    my $expression='$external::datestatus="'.$status.'";';
    $expression.='$external::gradestatus="'.$Apache::lonhomework::history{"resource.$id.solved"}.'";';
    &Apache::run::run($expression,$safeeval);
    if ( $status eq 'CLOSED' ) {
      my $bodytext=&Apache::lonxml::get_all_text("/part",$$parser[$#$parser]);
      if ( $target eq "web" ) {
	$result="<br />Part is not open to be viewed. It $accessmsg<br />";
      } elsif ( $target eq 'tex' ) {
	$result="\\vskip 0 mm Part is not open to be viewed. It $accessmsg \\\\";
      }
    } else {
      if ($target eq 'tex') {
	$result='\vskip 0 mm';
      }
    }
  }
  return $result;
}

sub end_part {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  &Apache::lonxml::debug("in end_part $target ");
  my $status=$Apache::inputtags::status['-1'];
  pop @Apache::inputtags::status;
  if ( $target eq 'meta' ) { return ''; }
  if ( $target eq 'grade' && $status eq 'CAN_ANSWER') {
    return &Apache::inputtags::grade;
  }
  if ($target eq 'web') {
    return &Apache::inputtags::gradestatus($Apache::inputtags::part);
  }
  return '';
}

sub start_preduedate {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer') {
    if ($Apache::inputtags::status['-1'] ne 'CAN_ANSWER' &&
	$Apache::inputtags::status['-1'] ne 'CANNOT_ANSWER' ) {
      &Apache::lonxml::get_all_text("/preduedate",$$parser[$#$parser]);
    }
  }
  return '';
}

sub end_preduedate {
  return '';
}

sub start_postanswerdate {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  if ($target eq 'web' || $target eq 'grade') {
    if ($Apache::inputtags::status['-1'] ne 'SHOW_ANSWER') {
      &Apache::lonxml::get_all_text("/postanswerdate",$$parser[$#$parser]);
    }
  }
  return '';
}

sub end_postanswerdate {
  return '';
}

sub start_notsolved {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer') {
    my $gradestatus=$Apache::lonhomework::history{"resource.$Apache::inputtags::part.solved"};
    &Apache::lonxml::debug("not solved has :$gradestatus:");
    if ($gradestatus =~ /^correct/) {
      &Apache::lonxml::debug("skipping");
      &Apache::lonxml::get_all_text("/notsolved",$$parser[$#$parser]);
    }
  }
  return '';
}

sub end_notsolved {
  return '';
}

sub start_solved {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  if ($target eq 'web' || $target eq 'grade' || $target eq 'answer') {
    my $gradestatus=$Apache::lonhomework::history{"resource.$Apache::inputtags::part.solved"};
    if ($gradestatus !~ /^correct/) {
      &Apache::lonxml::get_all_text("/solved",$$parser[$#$parser]);
    }
  }
  return '';
}

sub end_solved {
  return '';
}

sub start_startouttext {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my @result=(''.'');
  if ($target eq 'edit' || $target eq 'modified' ) { @result=('','no'); }
  return (@result);
}
sub end_startouttext {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result='';
  my $text='';

  if ($target eq 'edit') {
    $text=&Apache::lonxml::get_all_text("endouttext",$$parser[-1]);
    $result.=&Apache::edit::start_table($token)."<tr><td>Text Block</td>
<td>Delete:".
  &Apache::edit::deletelist($target,$token)
    ."</td>
<td>".
  &Apache::edit::insertlist($target,$token).
    &Apache::edit::end_row().&Apache::edit::start_spanning_row()."\n".
      &Apache::edit::editfield($token->[1],$text,"",50,4);
  }
  if ($target eq 'modified') {
    $text=&Apache::lonxml::get_all_text("endouttext",$$parser['-1']);
    $result='<startouttext />'.&Apache::edit::modifiedfield();
  }
  if ($target eq 'tex') {
      $result .= '\noindent ';
  }
  return $result;
}
sub start_endouttext {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my $result='';
  if ($target eq "edit" ) { $result="</td></tr>".&Apache::edit::end_table()."\n"; }
  if ($target eq "modified") { $result='<endouttext />'; }
  return $result;
}
sub end_endouttext {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
  my @result=('','');
  if ($target eq "edit" || $target eq 'modified') { @result=('','no'); }
  return (@result);
}
sub delete_startouttext {
  my ($target,$token,$tagstack,$parstack,$parser,$safeeval)=@_;
#  my $text=&Apache::lonxml::get_all_text("endouttext",$$parser['-1']);
  my $text=$$parser['-1']->get_text("/endouttext");
  my $ntoken=$$parser['-1']->get_token();
  &Apache::lonxml::debug("Deleting :$text: and :$ntoken->[0]:$ntoken->[1]:$ntoken->[2]: for startouttext");
  &Apache::lonxml::end_tag($tagstack,$parstack,$ntoken);
  # Deleting 2 parallel tag pairs, but we need the numbers later to look like 
  # they did the last time round
  &Apache::lonxml::increasedepth($ntoken);
  &Apache::lonxml::decreasedepth($ntoken);
  return 1;
}

1;
__END__
