# The LearningOnline Network with CAPA
# Metadata display handler
#
# $Id: lonmeta.pm,v 1.15 2002/03/29 18:44:41 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#
# (TeX Content Handler
#
# 05/29/00,05/30,10/11 Gerd Kortemeyer)
#
# 10/19,10/21,10/23,11/27,08/09/01,12/22,12/24,12/25 Gerd Kortemeyer

package Apache::lonmeta;

use strict;
use Apache::Constants qw(:common);
use Apache::lonnet();
use Apache::loncommon();

# ----------------------------------------- Fetch and evaluate dynamic metadata

sub dynamicmeta {
    my $url=&Apache::lonnet::declutter(shift);
    $url=~s/\.meta$//;
    my ($adomain,$aauthor)=($url=~/^(\w+)\/(\w+)\//);
    my $regexp=&Apache::lonnet::escape($url);
    $regexp=~s/(\W)/\\$1/g;
    $regexp='___'.$regexp.'___';
    my %evaldata=&Apache::lonnet::dump
                                     ('resevaldata',$adomain,$aauthor,$regexp);
    my %sum;
    my %cnt;
    my %listitems=('count'        => 'add',
                   'course'       => 'add',
                   'avetries'     => 'avg',
                   'stdno'        => 'add',
                   'difficulty'   => 'avg',
                   'clear'        => 'avg',
                   'technical'    => 'avg',
                   'helpful'      => 'avg',
                   'correct'      => 'avg',
                   'depth'        => 'avg',
                   'comments'     => 'app',
                   'usage'        => 'cnt'
                   );
    foreach (keys %evaldata) {
	$_=~/___(\w+)$/;
        if (defined($cnt{$1})) { $cnt{$1}++; } else { $cnt{$1}=1; }
        unless ($listitems{$1} eq 'app') {
            if (defined($sum{$1})) {
               $sum{$1}+=$evaldata{$_};
	    } else {
               $sum{$1}=$evaldata{$_};
	    }
        } else {
            if (defined($sum{$1})) {
               if ($evaldata{$_}) {
                  $sum{$1}.='<hr>'.$evaldata{$_};
	       }
 	    } else {
	       $sum{$1}=''.$evaldata{$_};
	    }
	}
    }
    my %returnhash=();
    foreach (keys %cnt) {
       if ($listitems{$_} eq 'avg') {
	   $returnhash{$_}=int(($sum{$_}/$cnt{$_})*100.0+0.5)/100.0;
       } elsif ($listitems{$_} eq 'cnt') {
           $returnhash{$_}=$cnt{$_};
       } else {
           $returnhash{$_}=$sum{$_};
       }
    }
    return %returnhash;
}

# -------------------------------------------------------------- Pretty display

sub evalgraph {
    my $value=shift;
    unless ($value) { return ''; }
    my $val=int($value*10.+0.5)-10;
    my $output='<table border=0 cellpadding=0 cellspacing=0><tr>';
    if ($val>=20) {
	$output.='<td width=20 bgcolor="#555555">&nbsp&nbsp;</td>';
    } else {
        $output.='<td width='.($val).' bgcolor="#555555">&nbsp;</td>'.
                 '<td width='.(20-$val).' bgcolor="#FF3333">&nbsp;</td>';
    }
    $output.='<td bgcolor="#FFFF33">&nbsp;</td>';
    if ($val>20) {
	$output.='<td width='.($val-20).' bgcolor="#33FF33">&nbsp;</td>'.
                 '<td width='.(40-$val).' bgcolor="#555555">&nbsp;</td>';
    } else {
       $output.='<td width=20 bgcolor="#555555">&nbsp&nbsp;</td>';
    }
    $output.='<td> ('.$value.') </td></tr></table>';
    return $output;
}

sub diffgraph {
    my $value=shift;
    unless ($value) { return ''; }
    my $val=int(40.0*$value+0.5);
    my @colors=('#FF9933','#EEAA33','#DDBB33','#CCCC33',
                '#BBDD33','#CCCC33','#DDBB33','#EEAA33');
    my $output='<table border=0 cellpadding=0 cellspacing=0><tr>';
    for (my $i=0;$i<8;$i++) {
	if ($val>$i*5) {
            $output.='<td width=5 bgcolor="'.$colors[$i].'">&nbsp;</td>';
        } else {
	    $output.='<td width=5 bgcolor="#555555">&nbsp;</td>';
	}
    }
    $output.='<td> ('.$value.') </td></tr></table>';
    return $output;
}

# ================================================================ Main Handler

sub handler {
  my $r=shift;
  my %content=();

# ----------------------------------------------------------- Set document type

  $r->content_type('text/html');
  $r->send_http_header;

  return OK if $r->header_only;

# ------------------------------------------------------------------- Read file

  my $uri=$r->uri;
  foreach (split(/\,/,&Apache::lonnet::metadata($uri,'keys'))) {
      $content{$_}=&Apache::lonnet::metadata($uri,$_);
  }

# ------------------------------------------------------------------ Hide stuff

  unless ($ENV{'user.adv'}) {
      foreach ('keywords','notes','abstract','subject') {
          $content{$_}='<i>- not displayed -</i>';
      }
  }

# --------------------------------------------------------------- Render Output

my $creationdate=localtime($content{'creationdate'});
my $lastrevisiondate=localtime($content{'lastrevisiondate'});
my $language=&Apache::loncommon::languagedescription($content{'language'});
my $mime=&Apache::loncommon::filedescription($content{'mime'}); 
my $disuri=&Apache::lonnet::declutter($uri);
  $disuri=~s/\.meta$//;
  $r->print(<<ENDHEAD);
<html><head><title>Catalog Information</title></head>
<body bgcolor="#FFFFFF">
<h1>Catalog Information</h1>
<h2>$content{'title'}</h2>
<h3><tt>$disuri</tt></h3>
<table cellspacing=2 border=0>
<tr><td bgcolor='#AAAAAA'>Author(s)</td>
<td bgcolor="#CCCCCC">$content{'author'}&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Subject</td>
<td bgcolor="#CCCCCC">$content{'subject'}&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Keyword(s)</td>
<td bgcolor="#CCCCCC">$content{'keywords'}&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Notes</td>
<td bgcolor="#CCCCCC">$content{'notes'}&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Abstract</td>
<td bgcolor="#CCCCCC">$content{'abstract'}&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>MIME Type</td>
<td bgcolor="#CCCCCC">$mime ($content{'mime'})&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Language</td>
<td bgcolor="#CCCCCC">$language&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Creation Date</td>
<td bgcolor="#CCCCCC">$creationdate&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>
Last Revision Date</td><td bgcolor="#CCCCCC">$lastrevisiondate&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Publisher/Owner</td>
<td bgcolor="#CCCCCC">$content{'owner'}&nbsp;</td></tr>
<tr><td bgcolor='#AAAAAA'>Copyright/Distribution</td>
<td bgcolor="#CCCCCC">$content{'copyright'}
</table>
ENDHEAD
  delete($content{'title'});
  delete($content{'author'});
  delete($content{'subject'});
  delete($content{'keywords'});
  delete($content{'notes'});
  delete($content{'abstract'});
  delete($content{'mime'});
  delete($content{'language'});
  delete($content{'creationdate'});
  delete($content{'lastrevisiondate'});
  delete($content{'owner'});
  delete($content{'copyright'});
  if ($ENV{'user.adv'}) {
# ------------------------------------------------------------ Dynamic Metadata
   $r->print(
   '<h3>Dynamic Metadata (updated periodically)</h3>Processing ...<br>');
   $r->rflush();
    my %items=(
 'count'      => 'Network-wide number of accesses (hits)',
 'course'     => 'Network-wide number of courses using resource',
 'usage'      => 'Number of resources using or importing resource',
 'clear'      => 'Material presented in clear way',
 'depth'      => 'Material covered with sufficient depth',
 'helpful'    => 'Material is helpful',
 'correct'    => 'Material appears to be correct',
 'technical'  => 'Resource is technically correct', 
 'avetries'   => 'Average number of tries till solved',
 'stdno'      => 'Total number of students who have worked on this problem',
 'difficulty' => 'Degree of difficulty');
   my %dynmeta=&dynamicmeta($uri);
   $r->print(
'</table><h4>Access and Usage Statistics</h4><table cellspacing=2 border=0>');
   foreach ('count','usage','course') {
       $r->print(
'<tr><td bgcolor="#AAAAAA">'.$items{$_}.'</td><td bgcolor="#CCCCCC">'.
$dynmeta{$_}."&nbsp;</td></tr>\n");
   }
       $r->print('</table>');
   if ($uri=~/\.(problem|exam|quiz|assess|survey|form)\.meta$/) {
      $r->print(
'<h4>Assessment Statistical Data</h4><table cellspacing=2 border=0>');
      foreach ('stdno','avetries') {
          $r->print(
'<tr><td bgcolor="#AAAAAA">'.$items{$_}.'</td><td bgcolor="#CCCCCC">'.
$dynmeta{$_}."&nbsp;</td></tr>\n");
      }
      foreach ('difficulty') {
         $r->print(
'<tr><td bgcolor="#AAAAAA">'.$items{$_}.'</td><td bgcolor="#CCCCCC">'.
&diffgraph($dynmeta{$_})."</td></tr>\n");
      }
      $r->print('</table>');    
   }
   $r->print('<h4>Evaluation Data</h4><table cellspacing=2 border=0>');
   foreach ('clear','depth','helpful','correct','technical') {
       $r->print(
'<tr><td bgcolor="#AAAAAA">'.$items{$_}.'</td><td bgcolor="#CCCCCC">'.
&evalgraph($dynmeta{$_})."</td></tr>\n");
   }    
   $r->print('</table>');
   $disuri=~/^(\w+)\/(\w+)\//;   
   if ((($ENV{'user.domain'} eq $1) && ($ENV{'user.name'} eq $2))
       || ($ENV{'user.role.ca./'.$1.'/'.$2})) {
      $r->print(
  '<h4>Evaluation Comments (visible to author and co-authors only)</h4>'.
      '<blockquote>'.$dynmeta{'comments'}.'</blockquote>');      
   }
# ------------------------------------------------------------- All other stuff
   $r->print(
 '<h3>Additional Metadata (non-standard, parameters, exports)</h3>');
   foreach (sort keys %content) {
      my $name=$_;
      my $display=&Apache::lonnet::metadata($uri,$name.'.display');
      unless ($display) { $display=$name; };
      my $otherinfo='';
      foreach ('name','part','type','default') {
          if (defined(&Apache::lonnet::metadata($uri,$name.'.'.$_))) {
             $otherinfo.=' '.$_.'='.
		 &Apache::lonnet::metadata($uri,$name.'.'.$_).'; ';
          }
      }
      $r->print('<b>'.$display.':</b> '.$content{$name});
      if ($otherinfo) {
         $r->print(' ('.$otherinfo.')');
      }
      $r->print("<br>\n");
   }
  }
  $r->print('</body></html>');
  return OK;
}

1;
__END__







